@@ -65,11 +65,13 @@ typedef struct
 static const FpIdEntry id_table[] = {
   { .vid = 0x10A5,  .pid = 0xFFE0,  },
   { .vid = 0x10A5,  .pid = 0xA305,  },
+  { .vid = 0x10A5,  .pid = 0xA306,  },
   { .vid = 0x10A5,  .pid = 0xDA04,  },
   { .vid = 0x10A5,  .pid = 0xD805,  },
   { .vid = 0x10A5,  .pid = 0xD205,  },
   { .vid = 0x10A5,  .pid = 0x9524,  },
   { .vid = 0x10A5,  .pid = 0x9544,  },
+  { .vid = 0x10A5,  .pid = 0xC844,  },
   /* terminating entry */
   { .vid = 0,  .pid = 0,  .driver_data = 0 },
 };
@@ -84,7 +86,7 @@ fpc_suspend_resume_cb (FpiUsbTransfer *transfer,
 
   fp_dbg ("%s current ssm state: %d", G_STRFUNC, ssm_state);
 
-  if (ssm_state == FP_CMD_SUSPENDED)
+  if (ssm_state == FPC_CMD_SUSPENDED)
     {
       if (error)
         fpi_ssm_mark_failed (transfer->ssm, error);
@@ -92,12 +94,12 @@ fpc_suspend_resume_cb (FpiUsbTransfer *transfer,
       fpi_device_suspend_complete (device, error);
       /* The resume handler continues to the next state! */
     }
-  else if (ssm_state == FP_CMD_RESUME)
+  else if (ssm_state == FPC_CMD_RESUME)
     {
       if (error)
         fpi_ssm_mark_failed (transfer->ssm, error);
       else
-        fpi_ssm_jump_to_state (transfer->ssm, FP_CMD_GET_DATA);
+        fpi_ssm_jump_to_state (transfer->ssm, FPC_CMD_GET_DATA);
 
       fpi_device_resume_complete (device, error);
     }
@@ -116,7 +118,7 @@ fpc_cmd_receive_cb (FpiUsbTransfer *transfer,
   if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) && (self->cmd_suspended))
     {
       g_error_free (error);
-      fpi_ssm_jump_to_state (transfer->ssm, FP_CMD_SUSPENDED);
+      fpi_ssm_jump_to_state (transfer->ssm, FPC_CMD_SUSPENDED);
       return;
     }
 
@@ -137,7 +139,7 @@ fpc_cmd_receive_cb (FpiUsbTransfer *transfer,
   fp_dbg ("%s current ssm request: %d state: %d", G_STRFUNC, data->request, ssm_state);
 
   /* clean cmd_ssm except capture command for suspend/resume case */
-  if (ssm_state != FP_CMD_SEND || data->request != FPC_CMD_ARM)
+  if (ssm_state != FPC_CMD_SEND || data->request != FPC_CMD_ARM)
     self->cmd_ssm = NULL;
 
   if (data->cmdtype == FPC_CMDTYPE_TO_DEVICE)
@@ -151,13 +153,13 @@ fpc_cmd_receive_cb (FpiUsbTransfer *transfer,
     }
   else if (data->cmdtype == FPC_CMDTYPE_TO_DEVICE_EVTDATA)
     {
-      if (ssm_state == FP_CMD_SEND)
+      if (ssm_state == FPC_CMD_SEND)
         {
           fpi_ssm_next_state (transfer->ssm);
           return;
         }
 
-      if (ssm_state == FP_CMD_GET_DATA)
+      if (ssm_state == FPC_CMD_GET_DATA)
         {
           fpc_cmd_response_t evt_data = {0};
           fp_dbg ("%s recv evt data length: %ld", G_STRFUNC, transfer->actual_length);
@@ -290,11 +292,11 @@ fpc_cmd_run_state (FpiSsm   *ssm,
 
   switch (fpi_ssm_get_cur_state (ssm))
     {
-    case FP_CMD_SEND:
+    case FPC_CMD_SEND:
       fpc_send_ctrl_cmd (dev);
       break;
 
-    case FP_CMD_GET_DATA:
+    case FPC_CMD_GET_DATA:
       transfer = fpi_usb_transfer_new (dev);
       transfer->ssm = ssm;
       fpi_usb_transfer_fill_bulk (transfer, EP_IN, EP_IN_MAX_BUF_SIZE);
@@ -305,7 +307,7 @@ fpc_cmd_run_state (FpiSsm   *ssm,
                                fpi_ssm_get_data (ssm));
       break;
 
-    case FP_CMD_SUSPENDED:
+    case FPC_CMD_SUSPENDED:
       transfer = fpi_usb_transfer_new (dev);
       transfer->ssm = ssm;
       fpi_usb_transfer_fill_control (transfer,
@@ -321,7 +323,7 @@ fpc_cmd_run_state (FpiSsm   *ssm,
                                fpc_suspend_resume_cb, NULL);
       break;
 
-    case FP_CMD_RESUME:
+    case FPC_CMD_RESUME:
       transfer = fpi_usb_transfer_new (dev);
       transfer->ssm = ssm;
       fpi_usb_transfer_fill_control (transfer,
@@ -352,21 +354,22 @@ fpc_sensor_cmd (FpiDeviceFpcMoc *self,
 
   data = g_memdup2 (cmd_data, sizeof (CommandData));
 
+  g_clear_object (&self->interrupt_cancellable);
+
   if (wait_data_delay)
     {
       self->cmd_data_timeout = 0;
-      g_set_object (&self->interrupt_cancellable, g_cancellable_new ());
+      self->interrupt_cancellable = g_cancellable_new ();
     }
   else
     {
       self->cmd_data_timeout = DATA_TIMEOUT;
-      g_clear_object (&self->interrupt_cancellable);
     }
 
   g_assert (self->cmd_ssm == NULL);
   self->cmd_ssm = fpi_ssm_new (FP_DEVICE (self),
                                fpc_cmd_run_state,
-                               FP_CMD_NUM_STATES);
+                               FPC_CMD_NUM_STATES);
 
   fpi_ssm_set_data (self->cmd_ssm, data, g_free);
   fpi_ssm_start (self->cmd_ssm, fpc_cmd_ssm_done);
@@ -741,7 +744,7 @@ fpc_enroll_update_cb (FpiDeviceFpcMoc *self,
     case FPC_ENROL_STATUS_COMPLETED:
       self->enroll_stage++;
       fpi_device_enroll_progress (FP_DEVICE (self), self->enroll_stage, NULL, NULL);
-      fpi_ssm_jump_to_state (self->task_ssm, FP_ENROLL_COMPLETE);
+      fpi_ssm_jump_to_state (self->task_ssm, FPC_ENROLL_COMPLETE);
       return;
 
     case FPC_ENROL_STATUS_IMAGE_TOO_SIMILAR:
@@ -760,7 +763,7 @@ fpc_enroll_update_cb (FpiDeviceFpcMoc *self,
               /* Used for customer enrollment scheme */
               if (self->enroll_stage >= (self->max_enroll_stage - self->max_immobile_stage))
                 {
-                  fpi_ssm_jump_to_state (self->task_ssm, FP_ENROLL_COMPLETE);
+                  fpi_ssm_jump_to_state (self->task_ssm, FPC_ENROLL_COMPLETE);
                   return;
                 }
               break;
@@ -778,7 +781,7 @@ fpc_enroll_update_cb (FpiDeviceFpcMoc *self,
       /* Used for customer enrollment scheme */
       if (self->enroll_stage >= (self->max_enroll_stage - self->max_immobile_stage))
         {
-          fpi_ssm_jump_to_state (self->task_ssm, FP_ENROLL_COMPLETE);
+          fpi_ssm_jump_to_state (self->task_ssm, FPC_ENROLL_COMPLETE);
           return;
         }
       break;
@@ -813,7 +816,7 @@ fpc_enroll_update_cb (FpiDeviceFpcMoc *self,
     }
   else
     {
-      fpi_ssm_jump_to_state (self->task_ssm, FP_ENROLL_CAPTURE);
+      fpi_ssm_jump_to_state (self->task_ssm, FPC_ENROLL_CAPTURE);
     }
 }
 
@@ -941,7 +944,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
 
   switch (fpi_ssm_get_cur_state (ssm))
     {
-    case FP_ENROLL_ENUM:
+    case FPC_ENROLL_ENUM:
       {
         FPC_FID_DATA pquery_data = {0};
         gsize query_data_len = 0;
@@ -966,7 +969,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_CREATE:
+    case FPC_ENROLL_CREATE:
       {
         recv_data_len = sizeof (FPC_BEGIN_ENROL);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -981,7 +984,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_CAPTURE:
+    case FPC_ENROLL_CAPTURE:
       {
         guint32 capture_id = FPC_CAPTUREID_RESERVED;
         fpi_device_report_finger_status_changes (device,
@@ -999,7 +1002,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_GET_IMG:
+    case FPC_ENROLL_GET_IMG:
       {
         cmd_data.cmdtype = FPC_CMDTYPE_TO_DEVICE_EVTDATA;
         cmd_data.request = FPC_CMD_GET_IMG;
@@ -1013,7 +1016,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_UPDATE:
+    case FPC_ENROLL_UPDATE:
       {
         recv_data_len = sizeof (FPC_ENROL);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1028,7 +1031,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_COMPLETE:
+    case FPC_ENROLL_COMPLETE:
       {
         recv_data_len = sizeof (FPC_END_ENROL);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1043,7 +1046,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_CHECK_DUPLICATE:
+    case FPC_ENROLL_CHECK_DUPLICATE:
       {
         recv_data_len = sizeof (FPC_IDENTIFY);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1058,7 +1061,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_BINDID:
+    case FPC_ENROLL_BINDID:
       {
         FPC_FID_DATA data = {0};
         gsize data_len = 0;
@@ -1112,7 +1115,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_COMMIT:
+    case FPC_ENROLL_COMMIT:
       {
         recv_data_len = sizeof (gint32);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1127,7 +1130,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_DICARD:
+    case FPC_ENROLL_DICARD:
       {
         cmd_data.cmdtype = FPC_CMDTYPE_TO_DEVICE;
         cmd_data.request = FPC_CMD_ABORT;
@@ -1140,7 +1143,7 @@ fpc_enroll_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_ENROLL_CLEANUP:
+    case FPC_ENROLL_CLEANUP:
       {
         if (self->do_cleanup == TRUE)
           {
@@ -1285,7 +1288,7 @@ fpc_verify_sm_run_state (FpiSsm *ssm, FpDevice *device)
 
   switch (fpi_ssm_get_cur_state (ssm))
     {
-    case FP_VERIFY_CAPTURE:
+    case FPC_VERIFY_CAPTURE:
       {
         guint32 capture_id = FPC_CAPTUREID_RESERVED;
         fpi_device_report_finger_status_changes (device,
@@ -1303,7 +1306,7 @@ fpc_verify_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_VERIFY_GET_IMG:
+    case FPC_VERIFY_GET_IMG:
       {
         cmd_data.cmdtype = FPC_CMDTYPE_TO_DEVICE_EVTDATA;
         cmd_data.request = FPC_CMD_GET_IMG;
@@ -1317,7 +1320,7 @@ fpc_verify_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_VERIFY_IDENTIFY:
+    case FPC_VERIFY_IDENTIFY:
       {
         gsize recv_data_len = sizeof (FPC_IDENTIFY);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1332,7 +1335,7 @@ fpc_verify_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_VERIFY_CANCEL:
+    case FPC_VERIFY_CANCEL:
       {
         cmd_data.cmdtype = FPC_CMDTYPE_TO_DEVICE;
         cmd_data.request = FPC_CMD_ABORT;
@@ -1402,7 +1405,7 @@ fpc_clear_sm_run_state (FpiSsm *ssm, FpDevice *device)
 
   switch (fpi_ssm_get_cur_state (ssm))
     {
-    case FP_CLEAR_DELETE_DB:
+    case FPC_CLEAR_DELETE_DB:
       {
         if (self->dbid)
           {
@@ -1429,7 +1432,7 @@ fpc_clear_sm_run_state (FpiSsm *ssm, FpDevice *device)
       }
       break;
 
-    case FP_CLEAR_CREATE_DB:
+    case FPC_CLEAR_CREATE_DB:
       {
         if (self->dbid)
           {
@@ -1534,7 +1537,7 @@ fpc_init_sm_run_state (FpiSsm *ssm, FpDevice *device)
 
   switch (fpi_ssm_get_cur_state (ssm))
     {
-    case FP_INIT:
+    case FPC_INIT:
       cmd_data.cmdtype = FPC_CMDTYPE_TO_DEVICE_EVTDATA;
       cmd_data.request = FPC_CMD_INIT;
       cmd_data.value = 0x1;
@@ -1546,7 +1549,7 @@ fpc_init_sm_run_state (FpiSsm *ssm, FpDevice *device)
       fpc_sensor_cmd (self, FALSE, &cmd_data);
       break;
 
-    case FP_LOAD_DB:
+    case FPC_INIT_LOAD_DB:
       {
         gsize recv_data_len = sizeof (FPC_LOAD_DB);
         cmd_data.cmdtype = FPC_CMDTYPE_FROM_DEVICE;
@@ -1637,11 +1640,13 @@ fpc_dev_probe (FpDevice *device)
     {
     case 0xFFE0:
     case 0xA305:
+    case 0xA306:
     case 0xD805:
     case 0xDA04:
     case 0xD205:
     case 0x9524:
     case 0x9544:
+    case 0xC844:
       self->max_enroll_stage = MAX_ENROLL_SAMPLES;
       break;
 
@@ -1678,7 +1683,7 @@ fpc_dev_open (FpDevice *device)
     }
 
   self->task_ssm = fpi_ssm_new (device, fpc_init_sm_run_state,
-                                FP_INIT_NUM_STATES);
+                                FPC_INIT_NUM_STATES);
 
   fpi_ssm_start (self->task_ssm, fpc_init_ssm_done);
 }
@@ -1690,6 +1695,7 @@ fpc_dev_close (FpDevice *device)
 
   fp_dbg ("%s enter -->", G_STRFUNC);
   g_clear_pointer (&self->dbid, g_free);
+  g_cancellable_cancel (self->interrupt_cancellable);
   g_clear_object (&self->interrupt_cancellable);
   fpc_dev_release_interface (self, NULL);
 }
@@ -1701,8 +1707,8 @@ fpc_dev_verify_identify (FpDevice *device)
 
   fp_dbg ("%s enter -->", G_STRFUNC);
   self->task_ssm = fpi_ssm_new_full (device, fpc_verify_sm_run_state,
-                                     FP_VERIFY_NUM_STATES,
-                                     FP_VERIFY_CANCEL,
+                                     FPC_VERIFY_NUM_STATES,
+                                     FPC_VERIFY_CANCEL,
                                      "verify_identify");
 
   fpi_ssm_start (self->task_ssm, fpc_verify_ssm_done);
@@ -1718,8 +1724,8 @@ fpc_dev_enroll (FpDevice *device)
   self->enroll_stage = 0;
   self->immobile_stage = 0;
   self->task_ssm = fpi_ssm_new_full (device, fpc_enroll_sm_run_state,
-                                     FP_ENROLL_NUM_STATES,
-                                     FP_ENROLL_DICARD,
+                                     FPC_ENROLL_NUM_STATES,
+                                     FPC_ENROLL_DICARD,
                                      "enroll");
 
   fpi_ssm_start (self->task_ssm, fpc_enroll_ssm_done);
@@ -1770,7 +1776,7 @@ fpc_dev_suspend (FpDevice *device)
     }
 
   g_assert (self->cmd_ssm);
-  g_assert (fpi_ssm_get_cur_state (self->cmd_ssm) == FP_CMD_GET_DATA);
+  g_assert (fpi_ssm_get_cur_state (self->cmd_ssm) == FPC_CMD_GET_DATA);
   self->cmd_suspended = TRUE;
   g_cancellable_cancel (self->interrupt_cancellable);
 }
@@ -1792,10 +1798,12 @@ fpc_dev_resume (FpDevice *device)
 
   g_assert (self->cmd_ssm);
   g_assert (self->cmd_suspended);
-  g_assert (fpi_ssm_get_cur_state (self->cmd_ssm) == FP_CMD_SUSPENDED);
+  g_assert (fpi_ssm_get_cur_state (self->cmd_ssm) == FPC_CMD_SUSPENDED);
   self->cmd_suspended = FALSE;
-  g_set_object (&self->interrupt_cancellable, g_cancellable_new ());
-  fpi_ssm_jump_to_state (self->cmd_ssm, FP_CMD_RESUME);
+
+  g_clear_object (&self->interrupt_cancellable);
+  self->interrupt_cancellable = g_cancellable_new ();
+  fpi_ssm_jump_to_state (self->cmd_ssm, FPC_CMD_RESUME);
 }
 
 static void
@@ -1859,8 +1867,8 @@ fpc_dev_clear_storage (FpDevice *device)
 
   fp_dbg ("%s enter -->", G_STRFUNC);
   self->task_ssm = fpi_ssm_new_full (device, fpc_clear_sm_run_state,
-                                     FP_CLEAR_NUM_STATES,
-                                     FP_CLEAR_NUM_STATES,
+                                     FPC_CLEAR_NUM_STATES,
+                                     FPC_CLEAR_NUM_STATES,
                                      "Clear storage");
 
   fpi_ssm_start (self->task_ssm, fpc_clear_ssm_done);
